@@ -14,8 +14,7 @@
 ##
 ##  start: svd, random or given matrices A, B and C
 ##
-##  roxygen2::roxygenise("c:/Users/valen/OneDrive/MyRepo/R/rrcov3way",
-##      load_code=roxygen2:::load_installed, clean=TRUE)
+##  roxygen2::roxygenise("c:/Users/valen/OneDrive/MyRepo/R/rrcov3way", load_code=roxygen2:::load_installed)
 
 
 #'  Alternating Least Squares (ALS) for Candecomp/Parafac (CP)
@@ -51,11 +50,12 @@
 #' @return The result of the decomposition as a list with the following
 #'  elements:
 #'    \itemize{
+#'    \item \code{fit} Value of the loss function
+#'    \item \code{fp} Fit value expressed as a percentage
+#'    \item \code{ss} Sum of squares
 #'    \item \code{A} Component matrix for the A-mode
 #'    \item \code{B} Component matrix for the B-mode
 #'    \item \code{C} Component matrix for the C-mode
-#'    \item \code{f} Value of the loss function
-#'    \item \code{fp} Fit value expressed as a percentage
 #'    \item \code{iter} Number of iterations
 #'    \item \code{tripcos} Minimal triple cosine between two components
 #'        across the three component matrices, used to inspect degeneracy
@@ -161,153 +161,154 @@ cp_als <- function (X, n, m, p, ncomp, const="none", start="random",
 
     ftiter <- matrix(0, maxit/10, 2)
     mintripcos <- 0
-    ssx <- sum(Xa^2)
 
-    ## If start == "random" OR start == "svd" and n or m or p < r
-    ##      OR const[i]=="nonneg"
-    ## Random start (with orthonormalized component matrices),
-    ##  nonnegative if const == "nonneg"
-    A <- if(const[1] == "nonneg") matrix(runif(max(n,r) * r), max(n,r))[1:n, , drop=FALSE]
-         else                     pracma::orth(matrix(rnorm(max(n,r) * r), max(n,r)))[1:n, , drop=FALSE]
-    B <- if(const[2] == "nonneg") matrix(runif(max(m,r) * r), max(m,r))[1:m, , drop=FALSE]
-         else                     pracma::orth(matrix(rnorm(max(m,r) * r), max(m,r)))[1:m, , drop=FALSE]
-    C <- if(const[3] == "nonneg") matrix(runif(max(p,r) * r), max(p,r))[1:p, , drop=FALSE]
-         else                     pracma::orth(matrix(rnorm(max(p,r) * r), max(p,r)))[1:p, , drop=FALSE]
+        ssx <- sum(Xa^2)
 
-    if(is.list(start)) {
-        A <- start$A
-        B <- start$B
-        C <- start$C
-    }else if(start == "svd") {
-        if(n >= r && const[1] != "nonneg")
-            A <- eigen(Xa %*% t(Xa))$vectors[, 1:r, drop=FALSE]
-        if(m >= r && const[2] != "nonneg")
-            B <- eigen(Xb %*% t(Xb))$vectors[, 1:r, drop=FALSE]
-        if(p >= r && const[3] != "nonneg")
-            C <- eigen(Xc %*% t(Xc))$vectors[, 1:r, drop=FALSE]
-    }
+        ## If start == "random" OR start == "svd" and n or m or p < r
+        ##      OR const[i]=="nonneg"
+        ## Random start (with orthonormalized component matrices),
+        ##  nonnegative if const == "nonneg"
+        A <- if(const[1] == "nonneg") matrix(runif(max(n,r) * r), max(n,r))[1:n, , drop=FALSE]
+             else                     pracma::orth(matrix(rnorm(max(n,r) * r), max(n,r)))[1:n, , drop=FALSE]
+        B <- if(const[2] == "nonneg") matrix(runif(max(m,r) * r), max(m,r))[1:m, , drop=FALSE]
+             else                     pracma::orth(matrix(rnorm(max(m,r) * r), max(m,r)))[1:m, , drop=FALSE]
+        C <- if(const[3] == "nonneg") matrix(runif(max(p,r) * r), max(p,r))[1:p, , drop=FALSE]
+             else                     pracma::orth(matrix(rnorm(max(p,r) * r), max(p,r)))[1:p, , drop=FALSE]
 
-    f <- sum((Xa - tcrossprod(A, krp(C, B)))^2)
-    if(trace)
-        cat(paste("Candecomp/Parafac function value at Start is ", f, sep = " "), fill = TRUE)
-
-    fold <- f + 2 * conv * f
-    iter <- 0
-    BB <- t(B) %*% B
-    CC <- t(C) %*% C
-
-    while((fold - f > conv * f | iter < 2) & f > conv^2 & iter < maxit) {
-
-        fold <- f
-
-        ## Step 1: Update mode A
-        if(const[1] == "none") {                # A - no constraint
-            A <- Xa %*% krp(C, B) %*% solve(crossprod(C) * crossprod(B))
-        } else if (const[1] == "orth")          # A - orthogonality
-        {
-            svd <- svd(Xa %*% krp(C, B))
-            A <- svd$u %*% t(svd$v)
-        } else if (const[1] == "nonneg")        # A - non-negativity
-        {
-            Atemp <- A
-            cob <- krp(C, B)
-            xtx <- crossprod(cob)
-            for(i in 1:n){
-                xty <- crossprod(cob, Xa[i,])
-                A[i, ] <- coef(nnls(cob, Xa[i,]))
-            }
-            if(any(colSums(A) == 0)){
-              A <- Atemp
-              stop(paste("Error in nonnegative LS for mode A at iter=", iter))
-            }
-        } else if(const[1] == "zerocor")        # A - zero correlations constraints
-        {
-            XF <- Xa %*% krp(C, B)
-            XFmean <- apply(XF, 2, mean)
-            svd <- svd(sweep(XF, 2, XFmean))
-            A <- sweep(svd$u %*% t(svd$v), 2, XFmean %*% solve(BB * CC), "+")
+        if(is.list(start)) {
+            A <- start$A
+            B <- start$B
+            C <- start$C
+        }else if(start == "svd") {
+            if(n >= r && const[1] != "nonneg")
+                A <- eigen(Xa %*% t(Xa))$vectors[, 1:r, drop=FALSE]
+            if(m >= r && const[2] != "nonneg")
+                B <- eigen(Xb %*% t(Xb))$vectors[, 1:r, drop=FALSE]
+            if(p >= r && const[3] != "nonneg")
+                C <- eigen(Xc %*% t(Xc))$vectors[, 1:r, drop=FALSE]
         }
-        AA <- t(A) %*% A
 
-        ## Step 2: Update mode B
-        if (const[2] == "none") {           # B - no constraint
-            B <- Xb %*% krp(C, A) %*% solve(crossprod(C) * crossprod(A))
-        } else if(const[2] == "orth")       # B - orthogonality
-        {
-            svd <- svd(Xb %*% krp(C, A))
-            B <- svd$u %*% t(svd$v)
-        } else if(const[2] == "nonneg")     # B - non-negativity
-        {
-            Btemp <- B
-            coa <- krp(C, A)
-            xtx <- crossprod(coa)
-            for(i in 1:m) {
-                xty <- crossprod(coa, Xb[i,])
-                B[i,] <- coef(nnls(coa, Xb[i,]))
-            }
-            if(any(colSums(B) == 0)){
-              B <- Btemp
-              stop(paste("Error in nonnegative LS for mode B at iter=", iter))
-            }
-        } else if(const[2] == "zerocor")   # B - zero correlations constraints
-        {
-            XF <- Xb %*% krp(C, A)
-            XFmean <- apply(XF, 2, mean)
-            svd <- svd(sweep(XF, 2, XFmean))
-            B <- sweep(svd$u %*% t(svd$v), 2, XFmean %*% solve(AA * CC), "+")
-        }
+        f <- sum((Xa - tcrossprod(A, krp(C, B)))^2)
+        if(trace)
+            cat(paste("\nCandecomp/Parafac function value at Start is ", f, sep = " "), fill = TRUE)
+
+        fold <- f + 2 * conv * f
+        iter <- 0
         BB <- t(B) %*% B
-
-        ## Step 3: Update mode C
-        if(const[3] == "none") {                # C - no constraint
-            C <- Xc %*% krp(B, A) %*% solve(crossprod(B) * crossprod(A))
-        }else if(const[3] == "orth")  {         # C - orthogonality
-            svd <- svd(Xc %*% krp(B, A))
-            C <- svd$u %*% t(svd$v)
-        }else if(const[3] == "nonneg") {        # C - non-negativity
-            Ctemp <- C
-            boa <- krp(B, A)
-            xtx <- crossprod(boa)
-            for(i in 1:p){
-                xty <- crossprod(boa, Xc[i,])
-                C[i, ] <- coef(nnls(boa, Xc[i,]))
-            }
-            if(any(colSums(C) == 0)){
-              C <- Ctemp
-              stop(paste("Error in nonnegative LS for mode C at iter=", iter))
-            }
-        }else if(const[3] == "zerocor") {       # C - zero correlation constraint
-            XF <- Xc %*% krp(B, A)
-            XFmean <- apply(XF, 2, mean)
-            svd <- svd(sweep(XF, 2, XFmean))
-            C <- sweep(svd$u %*% t(svd$v), 2, XFmean %*% solve(AA * BB), "+")
-        }
         CC <- t(C) %*% C
 
-        ## Step 4: Convergence check
-        f <- sum((Xa - tcrossprod(A, krp(C, B)))^2)
-        iter <- iter + 1
+        while((fold - f > conv * f | iter < 2) & f > conv^2 & iter < maxit) {
 
-        if(iter %% 10 == 0)
-        {
-            tripcos <- min(congruence(A, A) * congruence(B, B) * congruence(C, C))
-            if (iter == 10)
-                mintripcos <- tripcos
-            if (tripcos < mintripcos)
-                mintripcos <-  tripcos
-            if ((iter %% 1000) == 0 & trace)
-                cat(paste("Minimal Triple cosine =", tripcos), fill = TRUE)
+            fold <- f
 
-            ftiter[iter/10, ] <- c(f, tripcos)
+            ## Step 1: Update mode A
+            if(const[1] == "none") {                # A - no constraint
+                A <- Xa %*% krp(C, B) %*% solve(crossprod(C) * crossprod(B))
+            } else if (const[1] == "orth")          # A - orthogonality
+            {
+                svd <- svd(Xa %*% krp(C, B))
+                A <- svd$u %*% t(svd$v)
+            } else if (const[1] == "nonneg")        # A - non-negativity
+            {
+                Atemp <- A
+                cob <- krp(C, B)
+                xtx <- crossprod(cob)
+                for(i in 1:n){
+                    xty <- crossprod(cob, Xa[i,])
+                    A[i, ] <- coef(nnls(cob, Xa[i,]))
+                }
+                if(any(colSums(A) == 0)){
+                  A <- Atemp
+                  stop(paste("Error in nonnegative LS for mode A at iter=", iter))
+                }
+            } else if(const[1] == "zerocor")        # A - zero correlations constraints
+            {
+                XF <- Xa %*% krp(C, B)
+                XFmean <- apply(XF, 2, mean)
+                svd <- svd(sweep(XF, 2, XFmean))
+                A <- sweep(svd$u %*% t(svd$v), 2, XFmean %*% solve(BB * CC), "+")
+            }
+            AA <- t(A) %*% A
+
+            ## Step 2: Update mode B
+            if (const[2] == "none") {           # B - no constraint
+                B <- Xb %*% krp(C, A) %*% solve(crossprod(C) * crossprod(A))
+            } else if(const[2] == "orth")       # B - orthogonality
+            {
+                svd <- svd(Xb %*% krp(C, A))
+                B <- svd$u %*% t(svd$v)
+            } else if(const[2] == "nonneg")     # B - non-negativity
+            {
+                Btemp <- B
+                coa <- krp(C, A)
+                xtx <- crossprod(coa)
+                for(i in 1:m) {
+                    xty <- crossprod(coa, Xb[i,])
+                    B[i,] <- coef(nnls(coa, Xb[i,]))
+                }
+                if(any(colSums(B) == 0)){
+                  B <- Btemp
+                  stop(paste("Error in nonnegative LS for mode B at iter=", iter))
+                }
+            } else if(const[2] == "zerocor")   # B - zero correlations constraints
+            {
+                XF <- Xb %*% krp(C, A)
+                XFmean <- apply(XF, 2, mean)
+                svd <- svd(sweep(XF, 2, XFmean))
+                B <- sweep(svd$u %*% t(svd$v), 2, XFmean %*% solve(AA * CC), "+")
+            }
+            BB <- t(B) %*% B
+
+            ## Step 3: Update mode C
+            if(const[3] == "none") {                # C - no constraint
+                C <- Xc %*% krp(B, A) %*% solve(crossprod(B) * crossprod(A))
+            }else if(const[3] == "orth")  {         # C - orthogonality
+                svd <- svd(Xc %*% krp(B, A))
+                C <- svd$u %*% t(svd$v)
+            }else if(const[3] == "nonneg") {        # C - non-negativity
+                Ctemp <- C
+                boa <- krp(B, A)
+                xtx <- crossprod(boa)
+                for(i in 1:p){
+                    xty <- crossprod(boa, Xc[i,])
+                    C[i, ] <- coef(nnls(boa, Xc[i,]))
+                }
+                if(any(colSums(C) == 0)){
+                  C <- Ctemp
+                  stop(paste("Error in nonnegative LS for mode C at iter=", iter))
+                }
+            }else if(const[3] == "zerocor") {       # C - zero correlation constraint
+                XF <- Xc %*% krp(B, A)
+                XFmean <- apply(XF, 2, mean)
+                svd <- svd(sweep(XF, 2, XFmean))
+                C <- sweep(svd$u %*% t(svd$v), 2, XFmean %*% solve(AA * BB), "+")
+            }
+            CC <- t(C) %*% C
+
+            ## Step 4: Convergence check
+            f <- sum((Xa - tcrossprod(A, krp(C, B)))^2)
+            iter <- iter + 1
+
+            if(iter %% 10 == 0)
+            {
+                tripcos <- min(congruence(A, A) * congruence(B, B) * congruence(C, C))
+                if (iter == 10)
+                    mintripcos <- tripcos
+                if (tripcos < mintripcos)
+                    mintripcos <-  tripcos
+                if ((iter %% 1000) == 0 & trace)
+                    cat(paste("Minimal Triple cosine =", tripcos), fill = TRUE)
+
+                ftiter[iter/10, ] <- c(f, tripcos)
+            }
+
+            if(iter %% 50 == 0 && trace) {
+                cat(paste("f=", f, "after", iter, "iters; diff.=", (fold - f),
+                    sep = " "), fill = TRUE)
+            }
         }
 
-        if(iter %% 50 == 0 && trace) {
-            cat(paste("f=", f, "after", iter, "iters; diff.=", (fold - f),
-                sep = " "), fill = TRUE)
-        }
-    }
-
-    ftiter <- ftiter[1:iter/10, ]           # take only the first iter/10 rows
+    ftiter <- ftiter[1:(iter/10), ]           # take only the first iter/10 rows
     Rsq <- 1 - f/ssx
     fp <- 100 * Rsq
 
@@ -323,7 +324,7 @@ cp_als <- function (X, n, m, p, ncomp, const="none", start="random",
         cat(paste("Fit percentage is", round(fp, 2), "%", sep = " "), fill = TRUE)
     }
 
-    out <- list(A=A, B=B, C=C, f=f, fp=fp, ss=ssx, iter=iter, tripcos=tripcos,
-            mintripcos=mintripcos, ftiter=ftiter, const=const)
+    out <- list(fit=f, fp=fp, ss=ssx, A=A, B=B, C=C, iter=iter, 
+            tripcos=tripcos, mintripcos=mintripcos, ftiter=ftiter, const=const)
     out
 }
